#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <errno.h>
#include <Student.h>

using namespace std;

extern int errno;

// This system demonstrates the use of the socket() message passing
// IPC mechanism.  Sockets are different from pipes and shared memory 
// because they can operate BETWEEN computers using the network.
//
//
// The purpose of this system is demonstrate how to transmit different
// datatypes from a sender program to a receiver program using sockets.
// The system consists of two programs "sender" which sets up a TCP 
// port and accepts connections from clients.  And "receiver" which 
// connects to a TCP port to establish a connection with the server. 
//
// Once the connection between the sender and receiver is established,
// the sender sends:
// - a string
// - an int
// - a double
// - an object
//
// The receiver processes the data from the sender with the expectation
// that the data will be sent in the order in these comments.
int main(void)
{
  try {
    ////////////////////////////////////////////////////////////////////
    // Create a socket.  
    ////////////////////////////////////////////////////////////////////  
    int cs = socket(PF_INET,SOCK_STREAM,0);

    if (cs <= 0) {
      cerr << "clientr: Socket creation failed." << endl;
      exit(-1);
    }

    ////////////////////////////////////////////////////////////////////
    // Connect to server port
    ////////////////////////////////////////////////////////////////////
    sockaddr_in address;
    int addressSize = sizeof(sockaddr_in);
    address.sin_family=AF_INET;                         
    address.sin_port  = htons(15000); // <-------------------  port 15000
    inet_pton(AF_INET,"127.0.0.1",&address.sin_addr);  // <- IP 127.0.0.1 

    int status=0;
    status = connect(cs,(struct sockaddr*) &address, addressSize);

    if (status != 0) {
      cerr << "client:Connect failed, can't connect to port." << endl;
      cout << strerror(errno) << endl;
      exit(-1);
    }

    cout << "Server and client have established communication." << endl;

    ////////////////////////////////////////////////////////////////////
    // Send the data
    ////////////////////////////////////////////////////////////////////
    int bytesReceived;
    #define ACKLENGTH 4
    #define BYTEARRAYLENGTH 10000
    unsigned char byteArray[BYTEARRAYLENGTH];
    unsigned char ack[ACKLENGTH];

    // receive string & ack
    string theString; 
    bzero(byteArray, BYTEARRAYLENGTH);
    bytesReceived = recv(cs,byteArray,BYTEARRAYLENGTH,0);
    send(cs,ack,ACKLENGTH,0); //ack
    theString.assign((const char *) byteArray,bytesReceived);
    cout << "string received: " << theString << endl;

    // receive int & send ack
    int theInt = 0;
    bzero(byteArray, BYTEARRAYLENGTH);
    bytesReceived = recv(cs, byteArray, BYTEARRAYLENGTH, 0);
    ((unsigned char *)&theInt)[0]=byteArray[0];((unsigned char *)&theInt)[1]=byteArray[1];
    ((unsigned char *)&theInt)[2]=byteArray[2];((unsigned char *)&theInt)[3]=byteArray[3];
    ::send(cs, &ack, sizeof(ack), 0); // ack
    cout << "int received: " << theInt << endl;

    // receive double raw bytes & send ack
    double theDouble = 0.0;
    bzero(byteArray, BYTEARRAYLENGTH);
    bytesReceived = recv(cs, byteArray, BYTEARRAYLENGTH, 0);
    ((unsigned char *)&theDouble)[0]=byteArray[0];((unsigned char *)&theDouble)[1]=byteArray[1];
    ((unsigned char *)&theDouble)[2]=byteArray[2];((unsigned char *)&theDouble)[3]=byteArray[3];
    ((unsigned char *)&theDouble)[4]=byteArray[4];((unsigned char *)&theDouble)[5]=byteArray[5];
    ((unsigned char *)&theDouble)[6]=byteArray[6];((unsigned char *)&theDouble)[7]=byteArray[7];
    send(cs,ack,ACKLENGTH,0); //ack
    cout << "double received: " << theDouble << endl;
  
    // receive student & send ack
    Student theStudent;
    theStudent.receive(cs);
    cout << "studend received: " << theStudent.toString() << endl;

    // Cleanup
    close(cs);

  } catch (std::exception& e) {
    std::cerr << "main: " << e.what() << std::endl;
  }

  return 1;
}
