/*
 * sender.cpp
 *
 *  Created on: 
 *      Author: bdugan
 */

#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <errno.h>
#include <Student.h>

using namespace std;

// This system demonstrates the use of the socket() message passing
// IPC mechanism.  Sockets are different from pipes and shared memory 
// because they can operate BETWEEN computers using the network.
//
//
// The purpose of this system is demonstrate how to transmit different
// datatypes from a sender program to a receiver program using sockets.
// The system consists of two programs "sender" which sets up a TCP 
// port and accepts connections from clients.  And "receiver" which 
// connects to a TCP port to establish a connection with the server. 
//
// Once the connection between the sender and receiver is established,
// the sender sends:
// - a string
// - an int
// - a double
// - an object
//
// The receiver processes the data from the sender with the expectation
// that the data will be sent in the order in these comments.

int main(void)
{
  try {
    ////////////////////////////////////////////////////////////////////
    // Create a socket.  
    ////////////////////////////////////////////////////////////////////
    int s = socket(PF_INET,SOCK_STREAM,0);

    if (s <= 0) {
      cerr << "server: Socket creation failed." << endl;
      exit(-1);
    }

    ////////////////////////////////////////////////////////////////////
    // Create a port to listen to for client connections
    ////////////////////////////////////////////////////////////////////
    sockaddr_in address;
    int addressSize = sizeof(sockaddr);
    address.sin_family=AF_INET;
    address.sin_addr.s_addr = INADDR_ANY;
    address.sin_port  = htons(15000); // <---------------- port is 15000

    int status=0;
    status = bind(s,(struct sockaddr*) &address, addressSize);

    if (status != 0) {
      cerr << "server: Bind failed, unable to create port." << endl;
      exit(-1);
    }

    ////////////////////////////////////////////////////////////////////
    // Wait for client to connect to port
    ////////////////////////////////////////////////////////////////////
    cout << "server: Waiting for client to connect" << endl;
    listen(s,3);
    int cs = accept(s,(struct sockaddr*) &address,(socklen_t *) &addressSize);

    if (cs <= 0) {
      cerr << "server: Accept failed, client can't connect to port." << endl;
      cout << strerror(cs) << endl;
      exit(-1);
    }

    // We now have TWO sockets:
    //   s  - used to listen for connections from a receiver
    //   cs - used to send/recv messages between sender and receiver
    //        this socket was actually created by the receiver.
    //
    cout << "Server and client have established communication." << endl;

    ////////////////////////////////////////////////////////////////////
    // Send the data
    ////////////////////////////////////////////////////////////////////
    int bytesReceived;
    #define ACKLENGTH 4
    #define BYTEARRAYLENGTH 10000
    unsigned char byteArray[BYTEARRAYLENGTH];
    unsigned char ack[ACKLENGTH];

    // Convert to raw bytes & send & wait for ack
    string theString = "Come here Watson, I need you.";
    send(cs,theString.c_str(),theString.length(),0);
    bytesReceived =  recv(cs,ack,ACKLENGTH,0); //ack

    // Convert to raw bytes & send & wait for ack
    int theInt = 42424242;
    send(cs,&theInt,sizeof(int),0);
    bytesReceived =  recv(cs,ack,ACKLENGTH,0); //ack

    // Convert to raw bytes & send & wait for ack
    double theDouble = 42.0;
    send(cs,&theDouble,sizeof(int),0);
    bytesReceived =  recv(cs,ack,ACKLENGTH,0); //ack
  
    // Send a student & wait for ack
    Student theStudent;
    theStudent.first   = "Ben";             theStudent.last  = "Dugan";
    theStudent.address = "24 Davelin Road"; theStudent.phone = 5551212;
    theStudent.gpa     = 4.0;               theStudent.year  = 'F';
    theStudent.send(cs);

    // Cleanup
    close(cs);
    close(s);

  } catch (std::exception& e) {
    std::cerr << "main: " << e.what() << std::endl;
  }
}
